@@ -4,7 +4,7 @@ Autoconf-like configuration support.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -26,7 +26,9 @@ Autoconf-like configuration support.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/SConf.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/SConf.py 3266 2008/08/12 07:31:01 knight"
+
+import SCons.compat
 
 import os
 import re
@@ -39,16 +41,27 @@ import types
 import SCons.Action
 import SCons.Builder
 import SCons.Errors
+import SCons.Job
 import SCons.Node.FS
 import SCons.Taskmaster
 import SCons.Util
 import SCons.Warnings
 import SCons.Conftest
 
+from SCons.Debug import Trace
+
 # Turn off the Conftest error logging
 SCons.Conftest.LogInputFiles = 0
 SCons.Conftest.LogErrorMessages = 0
 
+# Set
+build_type = None
+build_types = ['clean', 'help']
+
+def SetBuildType(type):
+    global build_type
+    build_type = type
+
 # to be set, if we are in dry-run mode
 dryrun = 0
 
@@ -141,7 +154,6 @@ def _createSource( target, source, env ):
     fd.write(source[0].get_contents())
     fd.close()
 def _stringSource( target, source, env ):
-    import string
     return (str(target[0]) + ' <-\n  |' +
             string.replace( source[0].get_contents(),
                             '\n', "\n  |" ) )
@@ -158,12 +170,10 @@ class SConfBuildInfo(SCons.Node.FS.FileBuildInfo):
     """
     result = None # -> 0/None -> no error, != 0 error
     string = None # the stdout / stderr output when building the target
-    
-    def __init__(self, node, result, string, sig):
-        SCons.Node.FS.FileBuildInfo.__init__(self, node)
+
+    def set_build_result(self, result, string):
         self.result = result
         self.string = string
-        self.ninfo.bsig = sig
 
 
 class Streamer:
@@ -188,6 +198,11 @@ class Streamer:
         Return everything written to orig since the Streamer was created.
         """
         return self.s.getvalue()
+
+    def flush(self):
+        if self.orig:
+            self.orig.flush()
+        self.s.flush()
         
 
 class SConfBuildTask(SCons.Taskmaster.Task):
@@ -206,7 +221,7 @@ class SConfBuildTask(SCons.Taskmaster.Task):
         """
         if not isinstance(bi, SConfBuildInfo):
             SCons.Warnings.warn(SConfWarning,
-              "The stored build information has an unexpected class.")
+              "The stored build information has an unexpected class: %s" % bi.__class__)
         else:
             self.display("The original builder output was:\n" +
                          string.replace("  |" + str(bi.string),
@@ -229,7 +244,6 @@ class SConfBuildTask(SCons.Taskmaster.Task):
             except AttributeError:
                 # Earlier versions of Python don't have sys.excepthook...
                 def excepthook(type, value, tb):
-                    import traceback
                     traceback.print_tb(tb)
                     print type, value
             apply(excepthook, self.exc_info())
@@ -241,30 +255,39 @@ class SConfBuildTask(SCons.Taskmaster.Task):
         #       cached_error  is 1, if the node(s) are up_to_date, but the
         #                           build will fail
         #       cachable      is 0, if some nodes are not in our cache
-        is_up_to_date = 1
-        cached_error = 0
-        cachable = 1
+        T = 0
+        changed = False
+        cached_error = False
+        cachable = True
         for t in self.targets:
-            bi = t.get_stored_info()
+            if T: Trace('%s' % (t))
+            bi = t.get_stored_info().binfo
             if isinstance(bi, SConfBuildInfo):
+                if T: Trace(': SConfBuildInfo')
                 if cache_mode == CACHE:
                     t.set_state(SCons.Node.up_to_date)
+                    if T: Trace(': set_state(up_to-date)')
                 else:
-                    new_bsig = t.calc_signature(sconf_global.calc)
-                    if t.env.use_build_signature():
-                        old_bsig = bi.ninfo.bsig
-                    else:
-                        old_bsig = bi.ninfo.csig
-                    is_up_to_date = (is_up_to_date and new_bsig == old_bsig)
+                    if T: Trace(': get_state() %s' % t.get_state())
+                    if T: Trace(': changed() %s' % t.changed())
+                    if (t.get_state() != SCons.Node.up_to_date and t.changed()):
+                        changed = True
+                    if T: Trace(': changed %s' % changed)
                 cached_error = cached_error or bi.result
             else:
+                if T: Trace(': else')
                 # the node hasn't been built in a SConf context or doesn't
                 # exist
-                cachable = 0
-                is_up_to_date = 0
-        return (is_up_to_date, cached_error, cachable)
+                cachable = False
+                changed = ( t.get_state() != SCons.Node.up_to_date )
+                if T: Trace(': changed %s' % changed)
+        if T: Trace('\n')
+        return (not changed, cached_error, cachable)
 
     def execute(self):
+        if not self.targets[0].has_builder():
+            return
+
         sconf = sconf_global
 
         is_up_to_date, cached_error, cachable = self.collect_node_states()
@@ -277,11 +300,13 @@ class SConfBuildTask(SCons.Taskmaster.Task):
         if cached_error and is_up_to_date:
             self.display("Building \"%s\" failed in a previous run and all "
                          "its sources are up to date." % str(self.targets[0]))
-            self.display_cached_string(self.targets[0].get_stored_info())
+            binfo = self.targets[0].get_stored_info().binfo
+            self.display_cached_string(binfo)
             raise SCons.Errors.BuildError # will be 'caught' in self.failed
         elif is_up_to_date:            
             self.display("\"%s\" is up to date." % str(self.targets[0]))
-            self.display_cached_string(self.targets[0].get_stored_info())
+            binfo = self.targets[0].get_stored_info().binfo
+            self.display_cached_string(binfo)
         elif dryrun:
             raise ConfigureDryRunError(self.targets[0])
         else:
@@ -301,21 +326,43 @@ class SConfBuildTask(SCons.Taskmaster.Task):
             except SystemExit:
                 exc_value = sys.exc_info()[1]
                 raise SCons.Errors.ExplicitExit(self.targets[0],exc_value.code)
-            except:
+            except Exception, e:
                 for t in self.targets:
-                    sig = t.calc_signature(sconf.calc)
-                    string = s.getvalue()
-                    binfo = SConfBuildInfo(t,1,string,sig)
-                    t.dir.sconsign().set_entry(t.name, binfo)
-                raise
+                    binfo = t.get_binfo()
+                    binfo.__class__ = SConfBuildInfo
+                    binfo.set_build_result(1, s.getvalue())
+                    sconsign_entry = SCons.SConsign.SConsignEntry()
+                    sconsign_entry.binfo = binfo
+                    #sconsign_entry.ninfo = self.get_ninfo()
+                    # We'd like to do this as follows:
+                    #    t.store_info(binfo)
+                    # However, we need to store it as an SConfBuildInfo
+                    # object, and store_info() will turn it into a
+                    # regular FileNodeInfo if the target is itself a
+                    # regular File.
+                    sconsign = t.dir.sconsign()
+                    sconsign.set_entry(t.name, sconsign_entry)
+                    sconsign.merge()
+                raise e
             else:
                 for t in self.targets:
-                    sig = t.calc_signature(sconf.calc)
-                    string = s.getvalue()
-                    binfo = SConfBuildInfo(t,0,string,sig)
-                    t.dir.sconsign().set_entry(t.name, binfo)
+                    binfo = t.get_binfo()
+                    binfo.__class__ = SConfBuildInfo
+                    binfo.set_build_result(0, s.getvalue())
+                    sconsign_entry = SCons.SConsign.SConsignEntry()
+                    sconsign_entry.binfo = binfo
+                    #sconsign_entry.ninfo = self.get_ninfo()
+                    # We'd like to do this as follows:
+                    #    t.store_info(binfo)
+                    # However, we need to store it as an SConfBuildInfo
+                    # object, and store_info() will turn it into a
+                    # regular FileNodeInfo if the target is itself a
+                    # regular File.
+                    sconsign = t.dir.sconsign()
+                    sconsign.set_entry(t.name, sconsign_entry)
+                    sconsign.merge()
 
-class SConf:
+class SConfBase:
     """This is simply a class to represent a configure context. After
     creating a SConf object, you can call any tests. After finished with your
     tests, be sure to call the Finish() method, which returns the modified
@@ -334,7 +381,7 @@ class SConf:
         e.g. custom_tests={'CheckPrivate':MyPrivateTest}, where MyPrivateTest
         defines a custom test.
         Note also the conf_dir and log_file arguments (you may want to
-        build tests in the BuildDir, not in the SourceDir)
+        build tests in the VariantDir, not in the SourceDir)
         """
         global SConfFS
         if not SConfFS:
@@ -356,16 +403,17 @@ class SConf:
         default_tests = {
                  'CheckFunc'          : CheckFunc,
                  'CheckType'          : CheckType,
+                 'CheckTypeSize'      : CheckTypeSize,
+                 'CheckDeclaration'   : CheckDeclaration,
                  'CheckHeader'        : CheckHeader,
                  'CheckCHeader'       : CheckCHeader,
                  'CheckCXXHeader'     : CheckCXXHeader,
                  'CheckLib'           : CheckLib,
-                 'CheckLibWithHeader' : CheckLibWithHeader
+                 'CheckLibWithHeader' : CheckLibWithHeader,
                }
         self.AddTests(default_tests)
         self.AddTests(custom_tests)
         self.confdir = SConfFS.Dir(env.subst(conf_dir))
-        self.calc = None
         if not config_h is None:
             config_h = SConfFS.File(config_h)
         self.config_h = config_h
@@ -373,10 +421,36 @@ class SConf:
 
     def Finish(self):
         """Call this method after finished with your tests:
-        env = sconf.Finish()"""
+                env = sconf.Finish()
+        """
         self._shutdown()
         return self.env
 
+    def Define(self, name, value = None, comment = None):
+        """
+        Define a pre processor symbol name, with the optional given value in the
+        current config header.
+
+        If value is None (default), then #define name is written. If value is not
+        none, then #define name value is written.
+        
+        comment is a string which will be put as a C comment in the
+        header, to explain the meaning of the value (appropriate C comments /* and
+        */ will be put automatically."""
+        lines = []
+        if comment:
+            comment_str = "/* %s */" % comment
+            lines.append(comment_str)
+
+        if value is not None:
+            define_str = "#define %s %s" % (name, value)
+        else:
+            define_str = "#define %s" % name
+        lines.append(define_str)
+        lines.append('')
+
+        self.config_h_text = self.config_h_text + string.join(lines, '\n')
+
     def BuildNodes(self, nodes):
         """
         Tries to build the given nodes immediately. Returns 1 on success,
@@ -394,6 +468,13 @@ class SConf:
         old_os_dir = os.getcwd()
         SConfFS.chdir(SConfFS.Top, change_os_dir=1)
 
+        # Because we take responsibility here for writing out our
+        # own .sconsign info (see SConfBuildTask.execute(), above),
+        # we override the store_info() method with a null place-holder
+        # so we really control how it gets written.
+        for n in nodes:
+            n.store_info = n.do_not_store_info
+
         ret = 1
 
         try:
@@ -557,7 +638,7 @@ class SConf:
     def AddTest(self, test_name, test_instance):
         """Adds test_class to this SConf instance. It can be called with
         self.test_name(...)"""
-        setattr(self, test_name, SConf.TestWrapper(test_instance, self))
+        setattr(self, test_name, SConfBase.TestWrapper(test_instance, self))
 
     def AddTests(self, tests):
         """Adds all the tests given in the tests dictionary to this SConf
@@ -597,7 +678,8 @@ class SConf:
             else:
                 _ac_config_logs[self.logfile] = None
                 log_mode = "w"
-            self.logstream = open(str(self.logfile), log_mode)
+            fp = open(str(self.logfile), log_mode)
+            self.logstream = SCons.Util.Unbuffered(fp)
             # logfile may stay in a build directory, so we tell
             # the build system not to override it with a eventually
             # existing file with the same name in the source directory
@@ -741,6 +823,12 @@ class CheckContext:
         # TODO: should use self.vardict for $CC, $CPPFLAGS, etc.
         return not self.TryBuild(self.env.Object, text, ext)
 
+    def RunProg(self, text, ext):
+        self.sconf.cached = 1
+        # TODO: should use self.vardict for $CC, $CPPFLAGS, etc.
+        st, out = self.TryRun(text, ext)
+        return not st, out
+
     def AppendLIBS(self, lib_name_list):
         oldLIBS = self.env.get( 'LIBS', [] )
         self.env.Append(LIBS = lib_name_list)
@@ -768,6 +856,19 @@ class CheckContext:
     #### End of stuff used by Conftest.py.
 
 
+def SConf(*args, **kw):
+    if kw.get(build_type, True):
+        kw['_depth'] = kw.get('_depth', 0) + 1
+        for bt in build_types:
+            try:
+                del kw[bt]
+            except KeyError:
+                pass
+        return apply(SConfBase, args, kw)
+    else:
+        return SCons.Util.Null()
+
+
 def CheckFunc(context, function_name, header = None, language = None):
     res = SCons.Conftest.CheckFunc(context, function_name, header = header, language = language)
     context.did_show_result = 1
@@ -779,6 +880,20 @@ def CheckType(context, type_name, includes = "", language = None):
     context.did_show_result = 1
     return not res
 
+def CheckTypeSize(context, type_name, includes = "", language = None, expect = None):
+    res = SCons.Conftest.CheckTypeSize(context, type_name,
+                                       header = includes, language = language, 
+                                       expect = expect)
+    context.did_show_result = 1
+    return res
+
+def CheckDeclaration(context, declaration, includes = "", language = None):
+    res = SCons.Conftest.CheckDeclaration(context, declaration,
+                                          includes = includes, 
+                                          language = language)
+    context.did_show_result = 1
+    return not res
+
 def createIncludesFromHeaders(headers, leaveLast, include_quotes = '""'):
     # used by CheckHeader and CheckLibWithHeader to produce C - #include
     # statements from the specified header (list)
