@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -21,9 +21,7 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Scanner/Dir.py 0.96.93.D001 2006/11/06 08:31:54 knight"
-
-import string
+__revision__ = "src/engine/SCons/Scanner/Dir.py 3266 2008/08/12 07:31:01 knight"
 
 import SCons.Node.FS
 import SCons.Scanner
@@ -94,6 +92,14 @@ def scan_in_memory(node, env, path=()):
     """
     "Scans" a Node.FS.Dir for its in-memory entries.
     """
-    entry_list = filter(do_not_scan, node.entries.keys())
+    try:
+        entries = node.entries
+    except AttributeError:
+        # It's not a Node.FS.Dir (or doesn't look enough like one for
+        # our purposes), which can happen if a target list containing
+        # mixed Node types (Dirs and Files, for example) has a Dir as
+        # the first entry.
+        return []
+    entry_list = filter(do_not_scan, entries.keys())
     entry_list.sort()
-    return map(lambda n, e=node.entries: e[n], entry_list)
+    return map(lambda n, e=entries: e[n], entry_list)
