@@ -6,7 +6,7 @@ files.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -28,7 +28,7 @@ files.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Script/SConscript.py 0.97.D001 2007/05/17 11:35:19 knight"
+__revision__ = "src/engine/SCons/Script/SConscript.py 3266 2008/08/12 07:31:01 knight"
 
 import SCons
 import SCons.Action
@@ -39,7 +39,6 @@ import SCons.Errors
 import SCons.Node
 import SCons.Node.Alias
 import SCons.Node.FS
-import SCons.Options
 import SCons.Platform
 import SCons.SConf
 import SCons.Script.Main
@@ -65,6 +64,9 @@ import UserList
 #CommandLineTargets = []
 #DefaultTargets = []
 
+class SConscriptReturn(Exception):
+    pass
+
 launch_dir = os.path.abspath(os.curdir)
 
 GlobalDict = None
@@ -125,6 +127,8 @@ class Frame:
         # make sure the sconscript attr is a Node.
         if isinstance(sconscript, SCons.Node.Node):
             self.sconscript = sconscript
+        elif sconscript == '-':
+            self.sconscript = None
         else:
             self.sconscript = fs.File(str(sconscript))
 
@@ -133,10 +137,11 @@ call_stack = []
 
 # For documentation on the methods in this file, see the scons man-page
 
-def Return(*vars):
+def Return(*vars, **kw):
     retval = []
     try:
-        for var in vars:
+        fvars = SCons.Util.flatten(vars)
+        for var in fvars:
             for v in string.split(var):
                 retval.append(call_stack[-1].globals[v])
     except KeyError, x:
@@ -147,6 +152,11 @@ def Return(*vars):
     else:
         call_stack[-1].retval = tuple(retval)
 
+    stop = kw.get('stop', True)
+
+    if stop:
+        raise SConscriptReturn
+
 
 stack_bottom = '% Stack boTTom %' # hard to define a variable w/this name :)
 
@@ -242,7 +252,10 @@ def _SConscript(fs, *files, **kw):
                     except KeyError:
                         pass
                     try:
-                        exec _file_ in call_stack[-1].globals
+                        try:
+                            exec _file_ in call_stack[-1].globals
+                        except SConscriptReturn:
+                            pass
                     finally:
                         if old_file is not None:
                             call_stack[-1].globals.update({__file__:old_file})
@@ -390,16 +403,16 @@ class SConsEnvironment(SCons.Environment.Base):
         if kw.get('exports'):
             exports.extend(self.Split(kw['exports']))
 
-        build_dir = kw.get('build_dir')
-        if build_dir:
+        variant_dir = kw.get('variant_dir') or kw.get('build_dir')
+        if variant_dir:
             if len(files) != 1:
                 raise SCons.Errors.UserError, \
-                    "Invalid SConscript() usage - can only specify one SConscript with a build_dir"
+                    "Invalid SConscript() usage - can only specify one SConscript with a variant_dir"
             duplicate = kw.get('duplicate', 1)
             src_dir = kw.get('src_dir')
             if not src_dir:
                 src_dir, fname = os.path.split(str(files[0]))
-                files = [os.path.join(str(build_dir), fname)]
+                files = [os.path.join(str(variant_dir), fname)]
             else:
                 if not isinstance(src_dir, SCons.Node.Node):
                     src_dir = self.fs.Dir(src_dir)
@@ -409,11 +422,11 @@ class SConsEnvironment(SCons.Environment.Base):
                 if fn.is_under(src_dir):
                     # Get path relative to the source directory.
                     fname = fn.get_path(src_dir)
-                    files = [os.path.join(str(build_dir), fname)]
+                    files = [os.path.join(str(variant_dir), fname)]
                 else:
                     files = [fn.abspath]
-                kw['src_dir'] = build_dir
-            self.fs.BuildDir(build_dir, src_dir, duplicate)
+                kw['src_dir'] = variant_dir
+            self.fs.VariantDir(variant_dir, src_dir, duplicate)
 
         return (files, exports)
 
@@ -469,7 +482,7 @@ class SConsEnvironment(SCons.Environment.Base):
 
     def GetOption(self, name):
         name = self.subst(name)
-        return SCons.Script.Main.ssoptions.get(name)
+        return SCons.Script.Main.GetOption(name)
 
     def Help(self, text):
         text = self.subst(text, raw=1)
@@ -525,7 +538,7 @@ class SConsEnvironment(SCons.Environment.Base):
 
     def SetOption(self, name, value):
         name = self.subst(name)
-        SCons.Script.Main.ssoptions.set(name, value)
+        SCons.Script.Main.SetOption(name, value)
 
 #
 #
