@@ -12,7 +12,7 @@ it goes here.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -34,7 +34,7 @@ it goes here.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Script/__init__.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Script/__init__.py 3266 2008/08/12 07:31:01 knight"
 
 import time
 start_time = time.time()
@@ -44,29 +44,33 @@ import string
 import sys
 import UserList
 
-# Special chicken-and-egg handling of the "--debug=memoizer"
-# and "--debug=nomemoizer" flags:
+# Special chicken-and-egg handling of the "--debug=memoizer" flag:
 #
 # SCons.Memoize contains a metaclass implementation that affects how
-# the other classes are instantiated.  The Memoizer handles optional
-# counting of the hits and misses by using a different, parallel set of
-# functions, so we don't slow down normal operation any more than we
-# have to.  We can also tell it disable memoization completely.
+# the other classes are instantiated.  The Memoizer may add shim methods
+# to classes that have methods that cache computed values in order to
+# count and report the hits and misses.
 #
-# If we wait to enable the counting or disable memoization completely
-# until we've parsed the command line options normally, it will be too
-# late, because the Memoizer will have already analyzed the classes
-# that it's Memoizing and bound the (non-counting) versions of the
-# functions.  So we have to use a special-case, up-front check for
-# the "--debug=memoizer" and "--debug=nomemoizer" flags and do what's
-# appropriate before we import any of the other modules that use it.
+# If we wait to enable the Memoization until after we've parsed the
+# command line options normally, it will be too late, because the Memoizer
+# will have already analyzed the classes that it's Memoizing and decided
+# to not add the shims.  So we use a special-case, up-front check for
+# the "--debug=memoizer" flag and enable Memoizer before we import any
+# of the other modules that use it.
+
 _args = sys.argv + string.split(os.environ.get('SCONSFLAGS', ''))
 if "--debug=memoizer" in _args:
     import SCons.Memoize
-    SCons.Memoize.EnableCounting()
-if "--debug=nomemoizer" in _args:
-    import SCons.Memoize
-    SCons.Memoize.DisableMemoization()
+    import SCons.Warnings
+    try:
+        SCons.Memoize.EnableMemoization()
+    except SCons.Warnings.Warning:
+        # Some warning was thrown (inability to --debug=memoizer on
+        # Python 1.5.2 because it doesn't have metaclasses).  Arrange
+        # for it to be displayed or not after warnings are configured.
+        import Main
+        exc_type, exc_value, tb = sys.exc_info()
+        Main.delayed_warnings.append((exc_type, exc_value))
 del _args
 
 import SCons.Action
@@ -77,8 +81,10 @@ import SCons.Options
 import SCons.Platform
 import SCons.Scanner
 import SCons.SConf
+import SCons.Subst
 import SCons.Tool
 import SCons.Util
+import SCons.Variables
 import SCons.Defaults
 
 import Main
@@ -100,24 +106,28 @@ BuildTask               = Main.BuildTask
 CleanTask               = Main.CleanTask
 QuestionTask            = Main.QuestionTask
 #PrintHelp               = Main.PrintHelp
-OptParser               = Main.OptParser
-SConscriptSettableOptions = Main.SConscriptSettableOptions
+#SConscriptSettableOptions = Main.SConscriptSettableOptions
 
-keep_going_on_error     = Main.keep_going_on_error
-print_dtree             = Main.print_dtree
-print_explanations      = Main.print_explanations
-print_includes          = Main.print_includes
-print_objects           = Main.print_objects
-print_time              = Main.print_time
-print_tree              = Main.print_tree
-memory_stats            = Main.memory_stats
-ignore_errors           = Main.ignore_errors
+AddOption               = Main.AddOption
+GetOption               = Main.GetOption
+SetOption               = Main.SetOption
+Progress                = Main.Progress
+GetBuildFailures        = Main.GetBuildFailures
+
+#keep_going_on_error     = Main.keep_going_on_error
+#print_dtree             = Main.print_dtree
+#print_explanations      = Main.print_explanations
+#print_includes          = Main.print_includes
+#print_objects           = Main.print_objects
+#print_time              = Main.print_time
+#print_tree              = Main.print_tree
+#memory_stats            = Main.memory_stats
+#ignore_errors           = Main.ignore_errors
 #sconscript_time         = Main.sconscript_time
 #command_time            = Main.command_time
 #exit_status             = Main.exit_status
 #profiling               = Main.profiling
-repositories            = Main.repositories
-#num_jobs                = Main.num_jobs     # settable by SConscript.SetJobs()
+#repositories            = Main.repositories
 
 #
 import SConscript
@@ -127,20 +137,33 @@ call_stack              = _SConscript.call_stack
 
 #
 Action                  = SCons.Action.Action
-BoolOption              = SCons.Options.BoolOption
+AddMethod               = SCons.Util.AddMethod
+AllowSubstExceptions    = SCons.Subst.SetAllowableExceptions
 Builder                 = SCons.Builder.Builder
 Configure               = _SConscript.Configure
-EnumOption              = SCons.Options.EnumOption
 Environment             = SCons.Environment.Environment
-ListOption              = SCons.Options.ListOption
-PackageOption           = SCons.Options.PackageOption
-PathOption              = SCons.Options.PathOption
+#OptParser               = SCons.SConsOptions.OptParser
+FindPathDirs            = SCons.Scanner.FindPathDirs
 Platform                = SCons.Platform.Platform
 Return                  = _SConscript.Return
 Scanner                 = SCons.Scanner.Base
 Tool                    = SCons.Tool.Tool
 WhereIs                 = SCons.Util.WhereIs
 
+#
+BoolVariable            = SCons.Variables.BoolVariable
+EnumVariable            = SCons.Variables.EnumVariable
+ListVariable            = SCons.Variables.ListVariable
+PackageVariable         = SCons.Variables.PackageVariable
+PathVariable            = SCons.Variables.PathVariable
+
+# Deprecated names that will go away some day.
+BoolOption              = SCons.Options.BoolOption
+EnumOption              = SCons.Options.EnumOption
+ListOption              = SCons.Options.ListOption
+PackageOption           = SCons.Options.PackageOption
+PathOption              = SCons.Options.PathOption
+
 # Action factories.
 Chmod                   = SCons.Defaults.Chmod
 Copy                    = SCons.Defaults.Copy
@@ -249,7 +272,10 @@ def HelpFunction(text):
 sconscript_reading = 0
 
 #
-def Options(files=None, args=ARGUMENTS):
+def Variables(files=[], args=ARGUMENTS):
+    return SCons.Variables.Variables(files, args)
+
+def Options(files=[], args=ARGUMENTS):
     return SCons.Options.Options(files, args)
 
 # The list of global functions to add to the SConscript name space
@@ -263,12 +289,10 @@ GlobalDefaultEnvironmentFunctions = [
     'Exit',
     'Export',
     'GetLaunchDir',
-    'GetOption',
     'Help',
     'Import',
     #'SConscript', is handled separately, below.
     'SConscriptChdir',
-    'SetOption',
 
     # Methods from the Environment.Base class.
     'AddPostAction',
@@ -279,15 +303,20 @@ GlobalDefaultEnvironmentFunctions = [
     'CacheDir',
     'Clean',
     #The Command() method is handled separately, below.
+    'Decider',
     'Depends',
     'Dir',
     'NoClean',
+    'NoCache',
     'Entry',
     'Execute',
     'File',
     'FindFile',
+    'FindInstalledFiles',
+    'FindSourceFiles',
     'Flatten',
     'GetBuildPath',
+    'Glob',
     'Ignore',
     'Install',
     'InstallAs',
@@ -296,13 +325,16 @@ GlobalDefaultEnvironmentFunctions = [
     'ParseDepends',
     'Precious',
     'Repository',
+    'Requires',
     'SConsignFile',
     'SideEffect',
     'SourceCode',
     'SourceSignatures',
     'Split',
+    'Tag',
     'TargetSignatures',
     'Value',
+    'VariantDir',
 ]
 
 GlobalDefaultBuilders = [
@@ -330,10 +362,12 @@ GlobalDefaultBuilders = [
     'Tar',
     'TypeLibrary',
     'Zip',
+    'Package',
 ]
 
 for name in GlobalDefaultEnvironmentFunctions + GlobalDefaultBuilders:
     exec "%s = _SConscript.DefaultEnvironmentCall(%s)" % (name, repr(name))
+del name
 
 # There are a handful of variables that used to live in the
 # Script/SConscript.py module that some SConscript files out there were
@@ -345,6 +379,10 @@ for name in GlobalDefaultEnvironmentFunctions + GlobalDefaultBuilders:
 # this way by hanging some attributes off the "SConscript" object here.
 SConscript = _SConscript.DefaultEnvironmentCall('SConscript')
 
+# Make SConscript look enough like the module it used to be so
+# that pychecker doesn't barf.
+SConscript.__name__ = 'SConscript'
+
 SConscript.Arguments = ARGUMENTS
 SConscript.ArgList = ARGLIST
 SConscript.BuildTargets = BUILD_TARGETS
