@@ -8,7 +8,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,7 +30,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Tool/aixlink.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Tool/aixlink.py 3266 2008/08/12 07:31:01 knight"
 
 import os
 import os.path
@@ -44,7 +44,7 @@ cplusplus = __import__('c++', globals(), locals(), [])
 
 def smart_linkflags(source, target, env, for_signature):
     if cplusplus.iscplusplus(source):
-        build_dir = env.subst('$BUILDDIR')
+        build_dir = env.subst('$BUILDDIR', target=target, source=source)
         if build_dir:
             return '-qtempinc=' + os.path.join(build_dir, 'tempinc')
     return ''
