@@ -10,7 +10,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -32,7 +32,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/linkloc.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Tool/linkloc.py 3266 2008/08/12 07:31:01 knight"
 
 import os.path
 import re
@@ -96,8 +96,8 @@ def generate(env):
 
     msvs_version = env.get('MSVS_VERSION')
     include_path, lib_path, exe_path = get_msvc_paths(env, version = msvs_version)
-    env['ENV']['LIB']            = lib_path
-    env['ENV']['PATH']           = exe_path
+    env['ENV']['LIB'] = lib_path
+    env.PrependENVPath('PATH', exe_path)
 
     addPharLapPaths(env)
 
