@@ -9,7 +9,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,7 +31,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/msvs.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Tool/msvs.py 3266 2008/08/12 07:31:01 knight"
 
 import base64
 import md5
@@ -205,7 +205,7 @@ class _DSPGenerator:
         if len(buildtarget) == 1:
             bt = buildtarget[0]
             buildtarget = []
-            for v in variants:
+            for _ in variants:
                 buildtarget.append(bt)
 
         if not env.has_key('outdir') or env['outdir'] == None:
@@ -1010,7 +1010,6 @@ class _GenerateV7DSW(_DSWGenerator):
             self.file.write('\tGlobalSection(ProjectConfiguration) = postSolution\n')
 
         for name in confkeys:
-            name = name
             variant = self.configs[name].variant
             platform = self.configs[name].platform
             if self.version_num >= 8.0:
@@ -1019,8 +1018,10 @@ class _GenerateV7DSW(_DSWGenerator):
                     self.file.write('\t\t%s.%s|%s.ActiveCfg = %s|%s\n'
                                     '\t\t%s.%s|%s.Build.0 = %s|%s\n'  % (guid,variant,platform,variant,platform,guid,variant,platform,variant,platform))
             else:
-                self.file.write('\t\t%s.%s.ActiveCfg = %s|%s\n'
-                                '\t\t%s.%s.Build.0 = %s|%s\n'  %(self.slnguid,variant,variant,platform,self.slnguid,variant,variant,platform))
+                for p in self.dspfiles:
+                    guid = _generateGUID(p, '')
+                    self.file.write('\t\t%s.%s.ActiveCfg = %s|%s\n'
+                                    '\t\t%s.%s.Build.0 = %s|%s\n'  %(guid,variant,variant,platform,guid,variant,variant,platform))
 
         self.file.write('\tEndGlobalSection\n')
 
@@ -1279,7 +1280,7 @@ def get_visualstudio8_suites():
     try:
         idk = SCons.Util.RegOpenKeyEx(SCons.Util.HKEY_LOCAL_MACHINE,
             r'Software\Microsoft\VisualStudio\8.0')
-        id = SCons.Util.RegQueryValueEx(idk, 'InstallDir')
+        SCons.Util.RegQueryValueEx(idk, 'InstallDir')
         editions = { 'PRO': r'Setup\VS\Pro' }       # ToDo: add standard and team editions
         edition_name = 'STD'
         for name, key_suffix in editions.items():
@@ -1297,7 +1298,7 @@ def get_visualstudio8_suites():
     try:
         idk = SCons.Util.RegOpenKeyEx(SCons.Util.HKEY_LOCAL_MACHINE,
             r'Software\Microsoft\VCExpress\8.0')
-        id = SCons.Util.RegQueryValueEx(idk, 'InstallDir')
+        SCons.Util.RegQueryValueEx(idk, 'InstallDir')
         suites.append('EXPRESS')
     except SCons.Util.RegError:
         pass
@@ -1386,41 +1387,64 @@ def get_msvs_install_dirs(version = None, vs8suite = None):
         # try and enumerate the installed versions of the .NET framework.
         contents = os.listdir(rv['FRAMEWORKDIR'])
         l = re.compile('v[0-9]+.*')
-        versions = []
-        for entry in contents:
-            if l.match(entry):
-                versions.append(entry)
+        installed_framework_versions = filter(lambda e, l=l: l.match(e), contents)
 
         def versrt(a,b):
             # since version numbers aren't really floats...
             aa = a[1:]
             bb = b[1:]
-            aal = aa.split('.')
-            bbl = bb.split('.')
-            c = int(bbl[0]) - int(aal[0])
-            if c == 0:
-                c = int(bbl[1]) - int(aal[1])
-                if c == 0:
-                    c = int(bbl[2]) - int(aal[2])
-            return c
+            aal = string.split(aa, '.')
+            bbl = string.split(bb, '.')
+            # sequence comparison in python is lexicographical
+            # which is exactly what we want.
+            # Note we sort backwards so the highest version is first.
+            return cmp(bbl,aal)
 
-        versions.sort(versrt)
+        installed_framework_versions.sort(versrt)
 
-        rv['FRAMEWORKVERSIONS'] = versions
-        # assume that the highest version is the latest version installed
-        rv['FRAMEWORKVERSION'] = versions[0]
+        rv['FRAMEWORKVERSIONS'] = installed_framework_versions
+
+        # TODO: allow a specific framework version to be set
+
+        # Choose a default framework version based on the Visual
+        # Studio version.
+        DefaultFrameworkVersionMap = {
+            '7.0'   : 'v1.0',
+            '7.1'   : 'v1.1',
+            '8.0'   : 'v2.0',
+            # TODO: Does .NET 3.0 need to be worked into here somewhere?
+        }
+        try:
+            default_framework_version = DefaultFrameworkVersionMap[version[:3]]
+        except (KeyError, TypeError):
+            pass
+        else:
+            # Look for the first installed directory in FRAMEWORKDIR that
+            # begins with the framework version string that's appropriate
+            # for the Visual Studio version we're using.
+            for v in installed_framework_versions:
+                if v[:4] == default_framework_version:
+                    rv['FRAMEWORKVERSION'] = v
+                    break
+
+        # If the framework version couldn't be worked out by the previous
+        # code then fall back to using the latest version of the .NET
+        # framework
+        if not rv.has_key('FRAMEWORKVERSION'):
+            rv['FRAMEWORKVERSION'] = installed_framework_versions[0]
 
     # .NET framework SDK install dir
-    try:
-        if rv.has_key('FRAMEWORKVERSION') and rv['FRAMEWORKVERSION'][:4] == 'v1.1':
-            key = r'Software\Microsoft\.NETFramework\sdkInstallRootv1.1'
-        else:
-            key = r'Software\Microsoft\.NETFramework\sdkInstallRoot'
-
-        (rv['FRAMEWORKSDKDIR'], t) = SCons.Util.RegGetValue(SCons.Util.HKEY_LOCAL_MACHINE,key)
-
-    except SCons.Util.RegError:
-        pass
+    if rv.has_key('FRAMEWORKVERSION'):
+        # The .NET SDK version used must match the .NET version used,
+        # so we deliberately don't fall back to other .NET framework SDK
+        # versions that might be present.
+        ver = rv['FRAMEWORKVERSION'][:4]
+        key = r'Software\Microsoft\.NETFramework\sdkInstallRoot' + ver
+        try:
+            (rv['FRAMEWORKSDKDIR'], t) = SCons.Util.RegGetValue(SCons.Util.HKEY_LOCAL_MACHINE,
+                key)
+        except SCons.Util.RegError:
+            pass
 
     # MS Platform SDK dir
     try:
@@ -1478,7 +1502,7 @@ def GenerateProject(target, source, env):
     builddspfile = target[0]
     dspfile = builddspfile.srcnode()
 
-    # this detects whether or not we're using a BuildDir
+    # this detects whether or not we're using a VariantDir
     if not dspfile is builddspfile:
         try:
             bdsp = open(str(builddspfile), "w+")
@@ -1705,9 +1729,9 @@ def generate(env):
     env['MSVSSCONS'] = '"%s" -c "%s"' % (python_executable, getExecScriptMain(env))
     env['MSVSSCONSFLAGS'] = '-C "${MSVSSCONSCRIPT.dir.abspath}" -f ${MSVSSCONSCRIPT.name}'
     env['MSVSSCONSCOM'] = '$MSVSSCONS $MSVSSCONSFLAGS'
-    env['MSVSBUILDCOM'] = '$MSVSSCONSCOM $MSVSBUILDTARGET'
-    env['MSVSREBUILDCOM'] = '$MSVSSCONSCOM $MSVSBUILDTARGET'
-    env['MSVSCLEANCOM'] = '$MSVSSCONSCOM -c $MSVSBUILDTARGET'
+    env['MSVSBUILDCOM'] = '$MSVSSCONSCOM "$MSVSBUILDTARGET"'
+    env['MSVSREBUILDCOM'] = '$MSVSSCONSCOM "$MSVSBUILDTARGET"'
+    env['MSVSCLEANCOM'] = '$MSVSSCONSCOM -c "$MSVSBUILDTARGET"'
     env['MSVSENCODING'] = 'Windows-1252'
 
     try:
@@ -1737,6 +1761,9 @@ def generate(env):
     env['SCONS_HOME'] = os.environ.get('SCONS_HOME')
 
 def exists(env):
+    if not env['PLATFORM'] in ('win32', 'cygwin'):
+        return 0
+
     try:
         v = SCons.Tool.msvs.get_visualstudio_versions()
     except (SCons.Util.RegError, SCons.Errors.InternalError):
@@ -1747,7 +1774,11 @@ def exists(env):
         if env.has_key('MSVS_VERSION'):
             version_num, suite = msvs_parse_version(env['MSVS_VERSION'])
         if version_num >= 7.0:
-            return env.Detect('devenv')
+            # The executable is 'devenv' in Visual Studio Pro,
+            # Team System and others.  Express Editions have different
+            # executable names.  Right now we're only going to worry
+            # about Visual C++ 2005 Express Edition.
+            return env.Detect('devenv') or env.Detect('vcexpress')
         else:
             return env.Detect('msdev')
     else:
