@@ -8,7 +8,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,10 +30,9 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/mwld.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Tool/mwld.py 3266 2008/08/12 07:31:01 knight"
 
 import SCons.Tool
-import SCons.Tool.mwcc
 
 
 def generate(env):
@@ -60,7 +59,8 @@ def generate(env):
 
 
 def exists(env):
-    return mwcc.set_versions(env)
+    import SCons.Tool.mwcc
+    return SCons.Tool.mwcc.set_vars(env)
 
 
 def shlib_generator(target, source, env, for_signature):
