@@ -9,7 +9,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,7 +31,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/sunc++.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Tool/sunc++.py 3266 2008/08/12 07:31:01 knight"
 
 import SCons.Util
 
@@ -42,21 +42,37 @@ cplusplus = __import__('c++', globals(), locals(), [])
 # use the package installer tool lslpp to figure out where cppc and what
 # version of it is installed
 def get_cppc(env):
-    cppcPath = env.get('CXX', None)
+    cxx = env.get('CXX', None)
+    if cxx:
+        cppcPath = os.path.dirname(cxx)
+    else:
+        cppcPath = None
+
     cppcVersion = None
 
     pkginfo = env.subst('$PKGINFO')
     pkgchk = env.subst('$PKGCHK')
 
-    for package in ['SPROcpl']:
-        cmd = "%s -l %s 2>/dev/null | grep '^ *VERSION:'" % (pkginfo, package)
-        line = os.popen(cmd).readline()
-        if line:
-            cppcVersion = line.split()[-1]
-            cmd = "%s -l %s 2>/dev/null | grep '^Pathname:.*/bin/CC$' | grep -v '/SC[0-9]*\.[0-9]*/'" % (pkgchk, package)
+    def look_pkg_db(pkginfo=pkginfo, pkgchk=pkgchk):
+        version = None
+        path = None
+        for package in ['SPROcpl']:
+            cmd = "%s -l %s 2>/dev/null | grep '^ *VERSION:'" % (pkginfo, package)
             line = os.popen(cmd).readline()
-            cppcPath = os.path.dirname(line.split()[-1])
-            break
+            if line:
+                version = line.split()[-1]
+                cmd = "%s -l %s 2>/dev/null | grep '^Pathname:.*/bin/CC$' | grep -v '/SC[0-9]*\.[0-9]*/'" % (pkgchk, package)
+                line = os.popen(cmd).readline()
+                if line:
+                    path = os.path.dirname(line.split()[-1])
+                    break
+
+        return path, version
+
+    path, version = look_pkg_db()
+    if path and version:
+        cppcPath, cppcVersion = path, version
+
     return (cppcPath, 'CC', 'CC', cppcVersion)
 
 def generate(env):
