@@ -5,7 +5,7 @@ The main package for the SCons software construction utility.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -27,14 +27,14 @@ The main package for the SCons software construction utility.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/__init__.py 0.97.D001 2007/05/17 11:35:19 knight"
+__revision__ = "src/engine/SCons/__init__.py 3266 2008/08/12 07:31:01 knight"
 
-__version__ = "0.97"
+__version__ = "1.0.0"
 
-__build__ = "D001"
+__build__ = "r3266"
 
-__buildsys__ = "roxbury"
+__buildsys__ = "bangkok"
 
-__date__ = "2007/05/17 11:35:19"
+__date__ = "2008/08/12 07:31:01"
 
 __developer__ = "knight"
